# coding:utf-8

"""
    Info print
    ~~~~~~~~~~
    
    Insert in every stdout.write() a info line from witch code line this print comes.
    Usefull to find debug print statements ;)
    
    WARNING: This is very slow and should only be used with the developer server ;)
    
    Simply put this two lines in your settings:
        ---------------------------------------------------------------------- 
        from django_tools.utils import info_print
        info_print.redirect_stdout()
        ----------------------------------------------------------------------
    
    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author:$

    :copyleft: 2009-2010 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys
import inspect
import warnings

MAX_FILEPATH_LEN = 66



class InfoStdout(object):
    """ Insert in every stdout.write() a info line from witch code line this print comes."""
    def __init__(self, orig_stdout):
        self.orig_stdout = orig_stdout
        self.old_fileinfo = None

    def write(self, txt):
        fileinfo = self._get_fileinfo()
        if fileinfo != self.old_fileinfo:
            self.orig_stdout.write("\n%s:\n%s" % (fileinfo, txt))
            self.old_fileinfo = fileinfo
        else:
            self.orig_stdout.write(txt)

    def flush(self):
        self.orig_stdout.flush()

    def _get_fileinfo(self):
        """ return fileinfo: Where from the announcement comes? """
        try:
            self_basename = os.path.basename(__file__)
            if self_basename.endswith(".pyc"):
                # cut: ".pyc" -> ".py"
                self_basename = self_basename[:-1]

            for stack_frame in inspect.stack():
                # go forward in the stack, to outside of this file.
                filename = stack_frame[1]
                lineno = stack_frame[2]
                if os.path.basename(filename) != self_basename:
                    break

            if len(filename) >= MAX_FILEPATH_LEN:
                filename = "...%s" % filename[-MAX_FILEPATH_LEN:]
            fileinfo = "%s line %s" % (filename, lineno)
        except Exception, e:
            fileinfo = "(inspect Error: %s)" % e

        return fileinfo



__redirected = False

def redirect_stdout():
    global __redirected

    if not __redirected:
        __redirected = True
        try:
            warnings.warn("Redirect stdout/stderr for info print!")
            orig_stdout = sys.stdout
            sys.stdout = InfoStdout(orig_stdout)
            orig_stderr = sys.stderr
            sys.stderr = InfoStdout(orig_stdout)
        except Exception, err:
            print "Error:", err



# coding: utf-8

"""
    need full validators
    ~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2010-2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import os
import re
import urlparse

if __name__ == "__main__":
    # For doctest only
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.conf import settings
from django.utils.encoding import smart_unicode
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator, URLValidator
from django.utils.translation import ugettext_lazy as _


# re from django.utils.translation.trans_real.accept_language_re
language_code_re = re.compile(r'^([A-Za-z]{1,8}(?:-[A-Za-z]{1,8})*)$')
validate_language_code = RegexValidator(
    language_code_re,
    _(u'Enter a valid language code (Accept-Language header format, see RFC2616)'),
    'invalid'
)


class ExistingDirValidator(object):
    """
    >>> settings.DEBUG = False
    
    >>> v = ExistingDirValidator()
    >>> v(settings.MEDIA_ROOT)
    >>> v("does/not/exist")
    Traceback (most recent call last):
        ...
    ValidationError: [u"Directory doesn't exist!"]
    
    >>> v("../")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Directory is not in base path!']
    
    >>> v("//")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Directory is not in base path!']
    
    
    >>> v = ExistingDirValidator("/")
    >>> v("/etc/default/")
    >>> v("var/log")
    """
    def __init__(self, base_path=settings.MEDIA_ROOT):
        self.base_path = os.path.normpath(os.path.abspath(base_path))

    def __call__(self, value):
        value = smart_unicode(value)

        abs_path = os.path.normpath(os.path.abspath(os.path.join(self.base_path, value)))

        if not abs_path.startswith(self.base_path):
            if settings.DEBUG:
                msg = _(u"Directory %r is not in base path ('%s')" % (abs_path, self.base_path))
            else:
                msg = _(u"Directory is not in base path!")
            raise ValidationError(msg)

        if not os.path.isdir(abs_path):
            if settings.DEBUG:
                msg = _(u"Directory %r doesn't exist!") % abs_path
            else:
                msg = _(u"Directory doesn't exist!")

            raise ValidationError(msg)



class URLValidator2(URLValidator):
    """
    A flexible version of the original django URLValidator ;)
    
    scheme://netloc/path?query#fragment
    
       
    >>> URLValidator2(allow_all_schemes=True, allow_netloc=False)
    Traceback (most recent call last):
        ...
    AssertionError: Can't allow schemes without netloc!
    
    
    >>> URLValidator2(allow_schemes=("http","ftp"), allow_all_schemes=True)
    Traceback (most recent call last):
        ...
    Warning: allow_schemes would be ignored, while allow_all_schemes==True!
  
    
    >>> URLValidator2(allow_schemes=("svn",))("svn://domain.tld")
    >>> URLValidator2(allow_schemes=("http","ftp"))("svn://domain.tld")
    Traceback (most recent call last):
        ...
    ValidationError: [u"The URL doesn't start with a allowed scheme."]
    
    
    >>> URLValidator2(allow_query=False)("http://www.domain.tld/without/query/")
    >>> URLValidator2(allow_query=False)("http://www.domain.tld/with/?query")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Enter a valid URL without a query.']
    
    
    >>> URLValidator2(allow_fragment=False)("http://www.domain.tld/without/fragment/")
    >>> URLValidator2(allow_fragment=False)("http://www.domain.tld/with/a/#fragment")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Enter a valid URL without a fragment.']
    
    
    >>> URLValidator2(verify_exists=True)("http://www.pylucid.org/") # Failed if pylucid.org is down ;)
    >>> URLValidator2(verify_exists=True)("http://www.domain.tld/valid/url/does/not/exist/")
    Traceback (most recent call last):
        ...
    ValidationError: [u'This URL appears to be a broken link.']
      
    
    To allow only local path, without protocol/domain do this:
    >>> URLValidator2(allow_schemes=None, allow_netloc=False)("/path/?query#fragment")
    >>> URLValidator2(allow_schemes=None, allow_netloc=False)("www.pylucid.org/path/?query#fragment") # see note below!
    >>> URLValidator2(allow_schemes=None, allow_netloc=False)("http://domain.tld/path/?query#fragment")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Please enter a local URL (without protocol/domain).']
    
    **Note:** Validating the network location (netloc):
    Following the syntax specifications in RFC 1808, urlparse recognizes a
    netloc only if it is properly introduced by ‘//’. Otherwise the input is
    presumed to be a relative URL and thus to start with a path component.
    See: http://docs.python.org/library/urlparse.html#urlparse.urlparse
    
    >>> URLValidator2(allow_schemes=None, allow_netloc=False)("www.pylucid.org/path?query#fragment")
    >>> URLValidator2(allow_schemes=None, allow_netloc=False)("//www.pylucid.org/path?query#fragment")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Please enter a local URL (without protocol/domain).']
    """
    regex = re.compile(r'^.+$', re.IGNORECASE)

    def __init__(self, allow_schemes=("http", "https"), allow_all_schemes=False, allow_netloc=True, allow_query=True, allow_fragment=True,
            verify_exists=False, validator_user_agent=settings.URL_VALIDATOR_USER_AGENT):
        super(URLValidator2, self).__init__()

        if __debug__ and (allow_schemes or allow_all_schemes) and not allow_netloc:
            raise AssertionError("Can't allow schemes without netloc!")

        if __debug__ and allow_schemes and allow_all_schemes:
            raise Warning("allow_schemes would be ignored, while allow_all_schemes==True!")

        self.verify_exists = verify_exists
        self.user_agent = validator_user_agent

        self.allow_schemes = allow_schemes or ()
        self.allow_all_schemes = allow_all_schemes
        self.allow_netloc = allow_netloc
        self.allow_query = allow_query
        self.allow_fragment = allow_fragment

    def __call__(self, value):
        value = smart_unicode(value)
        scheme, netloc, path, query, fragment = urlparse.urlsplit(value)

        if (scheme or netloc) and not self.allow_schemes and not self.allow_all_schemes and not self.allow_netloc:
            raise ValidationError(_(u"Please enter a local URL (without protocol/domain)."), code="local")

        if scheme:
            if not self.allow_all_schemes and scheme not in self.allow_schemes:
                raise ValidationError(_(u"The URL doesn't start with a allowed scheme."), "scheme")

        if netloc and not self.allow_netloc:
            raise ValidationError(_(u'Enter a valid URL without domain.'), code='netloc')

        if query and not self.allow_query:
            raise ValidationError(_(u'Enter a valid URL without a query.'), code='query')

        if fragment and not self.allow_fragment:
            raise ValidationError(_(u'Enter a valid URL without a fragment.'), code='fragment')

        if self.verify_exists:
            super(URLValidator2, self).__call__(value)



if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding: utf-8

"""
    need full model and form fields
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2010 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"


from django import forms
from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_tools import validators


class LanguageCodeFormField(forms.CharField):
    """
    Language Code form field in Accept-Language header format (RFC 2616)
    
    >>> LanguageCodeFormField().clean('en')
    u'en'
    
    >>> LanguageCodeFormField().clean('en-GB')
    u'en-GB'
    
    >>> LanguageCodeFormField().clean("this is wrong")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Enter a valid value.']
    
    >>> LanguageCodeFormField().clean(None)
    Traceback (most recent call last):
        ...
    ValidationError: [u'This field is required.']
    
    >>> LanguageCodeFormField(required=False).clean(None)
    u''
    """
    def __init__(self, *args, **kwargs):
        super(LanguageCodeFormField, self).__init__(*args, **kwargs)
        self.validators.append(validators.validate_language_code)


class LanguageCodeModelField(models.CharField):
    """
    >>> LanguageCodeModelField(max_length=20).run_validators('en-GB')
    
    >>> LanguageCodeModelField(max_length=20).run_validators("this is wrong")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Enter a valid language code (Accept-Language header format, see RFC2616)']
    """
    default_validators = [validators.validate_language_code]
    description = _("Language Code in Accept-Language header format defined in RFC 2616")

#------------------------------------------------------------------------------


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding: utf-8

import os
import sys

sys.path.insert(0, os.path.dirname(__file__))

DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ":memory:"
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',

    'django_tools',
    'django_tools.local_sync_cache',
    'django_tools.tests',
)
SITE_ID = 1



#!/usr/bin/env python
# coding: utf-8

"""
    upgrade packages in virtualenv
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    A simple commandline script for call "pip install ---upgrade XY" for every
    package thats installed in a virtualenv.
    
    Simply copy/symlink it into the root directory of your virtualenv and
    start it.
    
    You can select witch package type should be upgrade:
        * both: package + editables
        * only packages
        * only editables
        
    More information, call:
        ./upgrade_virtualenv --help
        
    This will be obsolete, if pip has a own upgrade command, see: 
        https://github.com/pypa/pip/issues/59
    
    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys
import subprocess
from optparse import OptionParser

if __name__ == "__main__":
    # precheck if we in a activated virtualenv
    # if not, the pip import can raise a ImportError, if pip not installed
    # in the globale python environment
    if not hasattr(sys, 'real_prefix'):
        print("")
        print("Error: It seems that we are not running in a activated virtualenv!")
        print("")
        print("Please activate your environment first, e.g:")
        print("\t...my_env$ source bin/activate")
        print("")
        sys.exit(-1)

import pkg_resources

from pip import locations
from pip.util import get_installed_distributions, get_terminal_size
import pip


class ColorOut(object):
    """
    Borrowed from Django:
    http://code.djangoproject.com/browser/django/trunk/django/utils/termcolors.py
    
    >>> c = ColorOut()
    >>> c.supports_colors()
    True
    >>> c.color_support = True
    >>> c.colorize('no color')
    'no color'
    >>> c.colorize('bold', opts=("bold",))
    '\\x1b[1mbold\\x1b[0m'
    >>> c.colorize("colors!", foreground="red", background="blue", opts=("bold", "blink"))
    '\\x1b[31;44;1;5mcolors!\\x1b[0m'
    """
    color_names = ('black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white')
    foreground_colors = dict([(color_names[x], '3%s' % x) for x in range(8)])
    background_colors = dict([(color_names[x], '4%s' % x) for x in range(8)])
    opt_dict = {'bold': '1', 'underscore': '4', 'blink': '5', 'reverse': '7', 'conceal': '8'}

    def __init__(self):
        self.color_support = self.supports_colors()

    def supports_colors(self):
        if sys.platform in ('win32', 'Pocket PC'):
            return False

        # isatty is not always implemented!
        if hasattr(sys.stdout, 'isatty') and sys.stdout.isatty():
            return True
        else:
            return False

    def colorize(self, text, foreground=None, background=None, opts=()):
        """
        Returns your text, enclosed in ANSI graphics codes.
        """
        if not self.color_support:
            return text

        code_list = []

        if foreground:
            code_list.append(self.foreground_colors[foreground])
        if background:
            code_list.append(self.background_colors[background])

        for option in opts:
            code_list.append(self.opt_dict[option])

        if not code_list:
            return text

        return "\x1b[%sm%s\x1b[0m" % (';'.join(code_list), text)
c = ColorOut()


def get_upgradeable():
    dependency_links = []
    for dist in pkg_resources.working_set:
        if dist.has_metadata('dependency_links.txt'):
            dependency_links.extend(dist.get_metadata_lines('dependency_links.txt'))

    print("\ndependency_links: %r\n" % dependency_links)

    packages = []
    editables = []
    for dist in get_installed_distributions(local_only=True):
        req = pip.FrozenRequirement.from_dist(dist, dependency_links=dependency_links)

        if not req.editable:
            packages.append(req.name)
        else:
            # FIXME: How can we get this needes information easier?
            raw_cmd = str(req)
            full_url = raw_cmd.split()[1]
            url, full_version = full_url.rsplit("@", 1)
            rev = full_version.rsplit("-", 1)[1]

            if rev != "dev":
                pip_url = "%s@%s#egg=%s" % (url, rev, req.name)
            else:
                pip_url = "%s#egg=%s" % (url, req.name)

            editables.append(pip_url)

    if not packages:
        print(c.colorize("Found no local packages.", foreground="blue", opts=("underscore",)))
    else:
        print(c.colorize("Found theses local packages:", foreground="green"))
        for package in packages:
            print("\t %s" % package)

    if not editables:
        print(c.colorize("Found no local editables.", foreground="blue", opts=("underscore",)))
    else:
        print(c.colorize("Found theses local editables:", foreground="green"))
        for editable in editables:
            print("\t %s" % editable)

    return packages, editables


def check_activation():
    print("")
    print("sys.real_prefix: %s" % c.colorize(sys.real_prefix, foreground="magenta"))
    print("sys.prefix: %s" % c.colorize(sys.prefix, foreground="green", opts=("bold",)))
    print("use pip from: %s" % c.colorize(os.path.dirname(pip.__file__), foreground="blue", opts=("bold",)))
    print("")


def check_pip_version():
    try:
        pkg_resources.require("pip >= 1.0.1")
    except pkg_resources.VersionConflict, err:
        print(c.colorize("Error: outdated pip version!", foreground="red", opts=("bold", "reverse")))
        print("Original error: %s" % err)
        print("")
        print(c.colorize("You should upgrade pip, e.g.:", foreground="blue"))
        print("\tpip install --upgrade pip")
        print("")
        if raw_input("still continue? (y/n) ").lower() not in ("y", "j"):
            print(c.colorize("Abort, ok.", foreground="blue"))
            sys.exit(-1)


def print_options(options):
    output = []
    if options.dryrun:
        output.append("dry-run is on")
    if options.verbose:
        output.append("pip verbose mode is on")
    output.append("log saved in '%s'" % options.logfile)

    print(c.colorize("used options:", opts=("underscore",)))
    for line in output:
        print(c.colorize("\t* %s" % line, foreground="magenta"))


def call_pip(options, *args):
    pip_executeable = os.path.join(locations.bin_py, "pip")
    cmd = [pip_executeable, "install", "--upgrade"]
    if options.verbose:
        cmd.append("--verbose")
    if options.logfile:
        cmd.append("--log=%s" % options.logfile)
    cmd += args
    print("-"*get_terminal_size()[0])
    print("run: %s" % c.colorize(" ".join(cmd), foreground="blue"))
    if not options.dryrun:
        subprocess.call(cmd)


def main():
    parser = OptionParser()
    parser.add_option("--dry-run",
                      action="store_true", dest="dryrun", default=False,
                      help="display only the pip commands and do nothing.")
    parser.add_option("--verbose",
                      action="store_true", dest="verbose", default=False,
                      help="Turn on pip verbose mode")
    parser.add_option("--log",
                      action="store", dest="logfile", default="upgrade_virtualenv.log",
                      help="Log file where complete pip output will be kept")

    options, args = parser.parse_args()
    #print options, args

    check_activation()
    check_pip_version()

    print_options(options)

    packages, editables = get_upgradeable()

    print("")
    print(c.colorize("Witch local virtualenv packages would you like to upgrade?", opts=("underscore",)))
    print("")
    print("(1) both: package + editables")
    print("(2) only packages")
    print("(3) only editables")

    print(c.colorize("\nWarning:", foreground="red") + " Start update only, if editables was detected successful!")
    print("\tMore info: https://github.com/pypa/pip/issues/319")

    try:
        choice = raw_input("\nPlease select (1/2/3):")
    except KeyboardInterrupt:
        print("")
        sys.exit()
    if choice not in ("1", "2", "3"):
        print(c.colorize("Abort, ok.", foreground="blue"))
        sys.exit()

    if choice in ("1", "2"):
        for package in packages:
            call_pip(options, package)

    if choice in ("1", "3"):
        for editable in editables:
            call_pip(options, "--editable", editable)

    print("-"*get_terminal_size()[0])

    print("")
    if choice == "1":
        print(c.colorize("Done, all packages + editables are updated.", foreground="blue"))
    elif choice == "2":
        print(c.colorize("Done, Packages, but not the editables are updated.", foreground="blue"))
    elif choice == "3":
        print(c.colorize("Done, editables, but not normal packages are updated.", foreground="blue"))
    print("")


if __name__ == "__main__":
    main()





# coding: utf-8

"""   
    template render
    ~~~~~~~~~~~~~~
    
    Some tools around template rendering
    
    :copyleft: 2009 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.template import Context, Template

def render_string_template(template, context):
    """
    render the given template string with the context and return it as a string
    
    >>> render_string_template("Foo {{ bar }}!", {"bar": "BAR"})
    u'Foo BAR!'
    """
    t = Template(template)
    c = Context(context)
    return t.render(c)


if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=False)
    print "DocTest end."


#coding:utf-8

"""
    Addon for django-tagging
    
    http://code.google.com/p/django-tagging/
    
    Some code parts from http://code.google.com/p/django-tagging-autocomplete/    
    
    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from tagging.fields import TagField
from django_tools.tagging_addon.widgets import jQueryTagFieldWidget


class jQueryTagModelField(TagField):
    """
    A model field for a django-tagging field.
    Use a own widget to display existing tags and make them clickable with jQuery. 
    """
    def formfield(self, **kwargs):
        # Use our own widget and give him access to the model class
        kwargs['widget'] = jQueryTagFieldWidget(self.model)
        return super(jQueryTagModelField, self).formfield(**kwargs)



# -*- coding: utf-8 -*-
"""
    threadlocals middleware
    ~~~~~~~~~~~~~~~~~~~~~~~

    make the request object everywhere available (e.g. in model instance). 

    based on: http://code.djangoproject.com/wiki/CookBookThreadlocalsAndUser
    
    Put this into your settings:   
    --------------------------------------------------------------------------
        MIDDLEWARE_CLASSES = (
            ...
            'django_tools.middlewares.ThreadLocal.ThreadLocalMiddleware',
            ...
        )
    --------------------------------------------------------------------------
    
    
    Usage:
    --------------------------------------------------------------------------
    from django_tools.middlewares import ThreadLocal
    
    # Get the current request object:
    request = ThreadLocal.get_current_request()
    
    # You can get the current user directy with:
    user = ThreadLocal.get_current_user()
    --------------------------------------------------------------------------
    

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author:$

    :copyleft: 2009 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details."""

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local


_thread_locals = local()


def get_current_request():
    """ returns the request object for this thead """
    return getattr(_thread_locals, "request", None)


def get_current_user():
    """ returns the current user, if exist, otherwise returns None """
    request = get_current_request()
    if request:
        return getattr(request, "user", None)


class ThreadLocalMiddleware(object):
    """ Simple middleware that adds the request object in thread local storage."""
    def process_request(self, request):
        _thread_locals.request = request



#coding:utf-8

"""
    Addon for django-tagging
    
    http://code.google.com/p/django-tagging/
    
    Some code parts from http://code.google.com/p/django-tagging-autocomplete/    
    
    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.forms.widgets import TextInput
from tagging.models import Tag
from django.template.loader import render_to_string


class jQueryTagFieldWidget(TextInput):
    """
    A widget for a django-tagging field. Display existing tags and make them
    clickable with jQuery.
    
    Needs as first initial argument the model class for Tag.objects.cloud_for_model()
    
    To change the "Tag cloud_for_model queryset filters" do this, e.g.:
    ---------------------------------------------------------------------------
    class FooBarForm(forms.ModelForm):
    
        def __init__(self, *args, **kwargs):
            super(FooBarForm, self).__init__(*args, **kwargs)
            
            site = self.initial.get("site", None) or self.data.get("site", None) or settings.SITE_ID
            
            # change the tag queryset filter:
            self.fields["tags"].widget.tag_queryset_filters = {
                "site": site
            }
            
            self._setup_tag_filter()
    
        class Meta:
            model = FooBarModel
    
    --------------------------------------------------------------------------- 
    """
    def __init__(self, tag_model, *args, **kwargs):
        self.tag_model = tag_model
        self.tag_queryset_filters = kwargs.pop("tag_queryset_filters", {})
        super(jQueryTagFieldWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        html = super(jQueryTagFieldWidget, self).render(name, value, attrs)
        tag_cloud = Tag.objects.cloud_for_model(self.tag_model, steps=2, filters=self.tag_queryset_filters)
        context = {
            "html": html,
            "tag_cloud": tag_cloud,
            "field_id": attrs['id'],
        }
        return render_to_string("tagging_addon/jQueryTagField.html", context)



#!/usr/bin/env python
# coding: utf-8

"""
    Show responses in webbrowser
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Helper functions for displaying test responses in webbrowser.

    :copyleft: 2009-2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from pprint import pformat
from xml.sax.saxutils import escape
import os
import tempfile
import webbrowser

from django.contrib import messages
from django.views.debug import get_safe_settings

from django_tools.utils.stack_info import get_stack_info


# Bug with Firefox under Ubuntu.
# http://www.python-forum.de/topic-11568.html
#webbrowser._tryorder.insert(0, 'epiphany') # Use Epiphany, if installed.

# Variable to save if the browser is opend in the past.
BROWSER_TRACEBACK_OPENED = False

RESPONSE_INFO_ATTR = (
    "content", "cookies", "request", "status_code", "_headers", "context",
)





def debug_response(response, browser_traceback=True, msg="", display_tb=True):
    """
    Display the response content with a error reaceback in a webbrowser.
    TODO: We should delete the temp files after viewing!
    """
    global BROWSER_TRACEBACK_OPENED
    if browser_traceback != True or BROWSER_TRACEBACK_OPENED == True:
        return
    # Save for the next traceback
    BROWSER_TRACEBACK_OPENED = True

    content = response.content.decode("utf-8")
    url = response.request["PATH_INFO"]

    stack_info = get_stack_info(filepath_filter="django_tools")
    stack_info.append(msg)
    if display_tb:
        print
        print "debug_response:"
        print "-" * 80
        print "\n".join(stack_info)
        print "-" * 80

    stack_info = escape("".join(stack_info))

    response_info = "<dl>\n"

    response_info += "\t<dt><h3>template</h3></dt>\n"
    if hasattr(response, "templates") and response.templates:
        try:
            templates = pformat([template.name for template in response.templates])
        except AttributeError:
            templates = "---"
    else:
        templates = "---"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % templates

    response_info += "\t<dt><h3>messages</h3></dt>\n"
    msg = messages.get_messages(response.request)
    if msg:
        msg = "".join(["%s\n" % x for x in msg])
    else:
        msg = "---"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % msg

    for attr in RESPONSE_INFO_ATTR:
        # FIXME: There must be exist a easier way to display the info
        response_info += "\t<dt><h3>%s</h3></dt>\n" % attr
        value = getattr(response, attr, "---")
        value = pformat(value)
        value = escape(value)
        response_info += "\t<dd><pre>%s</pre></dd>\n" % value

    response_info += "\t<dt><h3>settings</h3></dt>\n"
    response_info += "\t<dd><pre>%s</pre></dd>\n" % pformat(get_safe_settings())

    response_info += "</dl>\n"


    if "</body>" in content:
        info = (
            "\n<br /><hr />\n"
            "<h2>Unittest info</h2>\n"
            "<dl>\n"
            "<dt><h3>url:</h3></dt><dd>%s</dd>\n"
            "<dt><h3>traceback:</h3></dt><dd><pre>%s</pre></dd>\n"
            "<dt><h2>response info:</h2></dt>%s\n"
            "</dl>\n"
            "</body>"
        ) % (url, stack_info, response_info)
        content = content.replace("</body>", info)
    else:
        # Not a html page?
        content += "\n<pre>\n"
        content += "-" * 79
        content += (
            "\nUnittest info\n"
            "=============\n"
            "url: %s\n"
            "traceback:\n%s\n</pre>"
            "response info:\n%s\n"
        ) % (url, stack_info, response_info)


    fd, file_path = tempfile.mkstemp(prefix="PyLucid_unittest_", suffix=".html")
    os.write(fd, content.encode("utf-8"))
    os.close(fd)
    url = "file://%s" % file_path
    print "\nDEBUG html page in Browser! (url: %s)" % url
    try:
        webbrowser.open(url)
    except:
        pass
    #time.sleep(0.5)
    #os.remove(file_path)




# -*- coding: utf-8 -*-

"""
    some additional template filters
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: 2008-08-15 10:37:58 +0200 (Fr, 15 Aug 2008) $
    $Rev: 1728 $
    $Author: JensDiemer $

    :copyleft: 2007-2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.template.defaultfilters import stringfilter
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode


CHMOD_TRANS_DATA = (
    u"---", u"--x", u"-w-", u"-wx", u"r--", u"r-x", u"rw-", u"rwx",
)
def chmod_symbol(mod):
    """
    Transform a os.stat().st_mode octal value to a symbolic string.
    ignores meta infromation like SUID, SGID or the Sticky-Bit.
    e.g. 40755 -> rwxr-xr-x
    """
    try:
        mod = int(mod) # The django template engine gives always a unicode string
    except ValueError:
        return u""
    mod = mod & 0777 # strip "meta info"
    mod_string = u"%o" % mod

    return u''.join(CHMOD_TRANS_DATA[int(num)] for num in mod_string)
chmod_symbol.is_safe = True
chmod_symbol = stringfilter(chmod_symbol)


def get_oct(value):
    """
    Convert an integer number to an octal string.
    """
    try:
        return oct(value)
    except:
        return value
get_oct.is_safe = False


def human_duration(t):
    """
    Converts a time duration into a friendly text representation.
    >>> human_duration(0.01)
    u'10.0 ms'
    >>> human_duration(1)
    u'1.0 sec'
    >>> human_duration(65.5)
    u'1.1 min'
    >>> human_duration(3540)
    u'59.0 min'
    >>> human_duration(3541)
    u'1.0 h'
    """
    if t < 1:
        return _("%.1f ms") % round(t * 1000, 1)
    elif t > 60 * 59:
        return _("%.1f h") % round(t / 60.0 / 60.0, 1)
    elif t > 59:
        return _("%.1f min") % round(t / 60.0, 1)
    else:
        return _("%.1f sec") % t
human_duration.is_safe = True

if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=False)
    print "DocTest end."



# coding: utf-8

"""
    Test Local sync cache
    ~~~~~~~~~~~~~~~~~~~~~
    
    For more information look into DocString in local_sync_cache.py !
    
    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import unittest
import time
import pprint

if __name__ == "__main__":
    # run unittest directly
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django_tools.tests.test_settings"

from django.core import management
from django.core.cache import cache

from django_tools.local_sync_cache.local_sync_cache import LocalSyncCache
from django_tools.local_sync_cache.LocalSyncCacheMiddleware import LocalSyncCacheMiddleware


class LocalSyncCacheTest(unittest.TestCase):
    def setUp(self):
        LocalSyncCache.CACHES = []
        LocalSyncCache.INIT_COUNTER = {}
        LocalSyncCache._OWN_RESET_TIMES = {}
        cache.clear()

    def testBasic(self):
        c = LocalSyncCache(id="test1")
        c["key1"] = "value1"
        self.assertEqual(c, {'key1': 'value1'})

        cache.set("test1", time.time())

        c.check_state()
        self.assertEqual(c, {})

        cache_information = LocalSyncCache.get_cache_information()
        self.assertEqual(len(cache_information), 1)

    def testUniqueID(self):
        c1 = LocalSyncCache(id="test1")
        self.assertRaises(AssertionError, LocalSyncCache, id="test1")

    def testEmptyPformatCacheInfo(self):
        txt = LocalSyncCache.pformat_cache_information()
        self.assertEqual(txt, "")

    def testPformatCacheInfo(self):
        LocalSyncCache(id="FooBar")
        txt = LocalSyncCache.pformat_cache_information()
        self.assertTrue("id: FooBar" in txt)
        self.assertTrue("instance: {}" in txt)
        self.assertTrue("cleared: False" in txt)

    def testMulti(self):
        self.assertEqual(len(LocalSyncCache.CACHES), 0)
        c1 = LocalSyncCache(id="test1")
        self.assertEqual(len(LocalSyncCache.CACHES), 1)
        c2 = LocalSyncCache(id="test1", unique_ids=False)
        self.assertEqual(len(LocalSyncCache.CACHES), 2)
        c1["c1"] = "foo"
        c2["c2"] = "bar"
        self.assertEqual(c1, {'c1': 'foo'})
        self.assertEqual(c2, {'c2': 'bar'})

        c1.check_state()
        c2.check_state()

        self.assertEqual(c1, {'c1': 'foo'})
        self.assertEqual(c2, {'c2': 'bar'})

        c1.clear()
        self.assertEqual(c1, {})

        # In a "new request" all the same caches should be cleared
        c2.check_state()
        self.assertEqual(c2, {})

        cache_information = LocalSyncCache.get_cache_information()
        self.assertEqual(len(cache_information), 2)
#        for item in cache_information:
#            print item["instance"].id, item

    def testLocalSyncCacheMiddleware(self):
        middleware = LocalSyncCacheMiddleware()

        c1 = LocalSyncCache(id="testLocalSyncCacheMiddleware1")
        c2 = LocalSyncCache(id="testLocalSyncCacheMiddleware1", unique_ids=False)

        c3 = LocalSyncCache(id="testLocalSyncCacheMiddleware2")

        self.assertEqual(len(LocalSyncCache.CACHES), 3)

        c1["c1"] = "foo"
        c2["c2"] = "bar"
        c3["foo"] = "bar"

        middleware.process_request(None)

        self.assertEqual(c1, {'c1': 'foo'})
        self.assertEqual(c2, {'c2': 'bar'})
        self.assertEqual(c3, {'foo': 'bar'})

        c1.clear()
        self.assertEqual(c1, {})

        # In a "new request" all the same caches should be cleared
        middleware.process_request(None)
        self.assertEqual(c2, {})

        # Other caches should be not affected by clear()
        self.assertEqual(c3, {'foo': 'bar'})

        c1["c1"] = "foo2"
        c2["c2"] = "bar2"

        middleware.process_request(None)

        self.assertEqual(c1, {'c1': 'foo2'})
        self.assertEqual(c2, {'c2': 'bar2'})
        self.assertEqual(c3, {'foo': 'bar'})

        c2.clear()
        self.assertEqual(c2, {})

        # In a "new request" all the same caches should be cleared
        middleware.process_request(None)
        self.assertEqual(c1, {})

#        print LocalSyncCache.pformat_cache_information()
        cache_information = LocalSyncCache.get_cache_information()
        self.assertEqual(len(cache_information), 3)
        for item in cache_information:
            instance = item["instance"]
            self.assertEqual(instance.request_counter, 4)

            if instance.id == "testLocalSyncCacheMiddleware2":
                self.assertEqual(instance.own_clear_counter, 0)
                self.assertEqual(instance.ext_clear_counter, 0)
                self.assertEqual(instance, {'foo': 'bar'})
            else:
                self.assertEqual(instance.own_clear_counter, 1)
                self.assertEqual(instance.ext_clear_counter, 1)
                self.assertEqual(instance, {})


if __name__ == "__main__":
    # Run this unitest directly
#    management.call_command('test', "django_tools.tests.test_local_sync_cache.LocalSyncCacheTest",
#        verbosity=2,
#        failfast=True
#    )
    unittest.main()




# coding: utf-8

"""
    Local sync cache Middleware
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Calles check_state() in every existing LocalSyncCache instance.
    
    For more information look into DocString in local_sync_cache.py !
    
    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django_tools.local_sync_cache.local_sync_cache import LocalSyncCache

class LocalSyncCacheMiddleware(object):
    def process_request(self, request):
        for cache in LocalSyncCache.CACHES:
            cache.check_state()




# coding: utf-8

"""
    some decorators
    ~~~~~~~~~~~~~~~
    
    from PyLucid decorators.

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import sys
import warnings
from django.contrib import messages
try:
    from functools import wraps
except ImportError:
    from django.utils.functional import wraps  # Python 2.3, 2.4 fallback.


from django.conf import settings
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.contrib.auth.models import Permission
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpResponseRedirect, HttpRequest


def check_permissions(superuser_only, permissions=()):
    """
    Protect a view and limit it to users witch are log in and has the permissions.
    If the user is not log in -> Redirect him to a log in view with a next_url back to the requested page.
    
    TODO: Add a log entry, if user has not all permissions.
    
    Usage:
    --------------------------------------------------------------------------    
    @check_permissions(superuser_only=False, permissions=(u'appname.add_modelname', u'appname.change_modelname'))
    def my_view(request):
        ...
    --------------------------------------------------------------------------
    """
    assert isinstance(superuser_only, bool)
    assert isinstance(permissions, (list, tuple))

    def _inner(view_function):
        @wraps(view_function)
        def _check_permissions(request, *args, **kwargs):
            user = request.user

            if not user.is_authenticated():
                #FIXME: HttpResponseRedirect to admin login?
                msg = _("Permission denied for anonymous user. Please log in.")
                if settings.DEBUG: # Usefull??
                    warnings.warn(msg)
                raise PermissionDenied(msg)

            if not user.has_perms(permissions):
                msg = "User %r has not all permissions: %r (existing permissions: %r)" % (
                    user, permissions, user.get_all_permissions()
                )
                if settings.DEBUG: # Usefull??
                    warnings.warn(msg)
                raise PermissionDenied(msg)
            return view_function(request, *args, **kwargs)

        # Add superuser_only and permissions attributes, so they are accessible
        # Used to build the admin menu
        _check_permissions.superuser_only = superuser_only
        _check_permissions.permissions = permissions

        return _check_permissions
    return _inner


def render_to(template_name=None, debug=False, **response_kwargs):
    """
    Based on the decorators from django-annoying.

    Example:
 
        @render_to('foo/template.html')
        def PyLucidPluginFoo(request):
            bar = Bar.object.all()  
            return {'bar': bar}
        
    The view can also insert the template name in the context, e.g.:

        @render_to
        def PyLucidPluginFoo(request):
            bar = Bar.object.all()  
            return {'bar': bar, 'template_name': 'foo/template.html'}
    """
    def renderer(function):
        @wraps(function)
        def wrapper(*args, **kwargs):
            context = function(*args, **kwargs)

            request = None
            for arg in args:
                if isinstance(arg, HttpRequest):
                    request = arg

            if not request:
                raise AttributeError("Can't get request object from function arguments!")

            if not isinstance(context, dict):
                if debug:
                    msg = (
                        "renter_to info: %s (template: %r)"
                        " has not return a dict, has return: %r (%r)"
                    ) % (function.__name__, template_name, type(context), function.func_code)
                    messages.info(request, msg)
                return context

            template_name2 = context.pop('template_name', template_name)
            assert template_name2 != None, \
                ("Template name must be passed as render_to parameter"
                " or 'template_name' must be inserted into context!")

            response = render_to_response(template_name2, context, context_instance=RequestContext(request), **response_kwargs)

            if debug:
                messages.info(request, "render debug for %r (template: %r):" % (function.__name__, template_name2))
                messages.info(request, "local view context:", context)
                messages.info(request, "response:", response.content)

            return response
        return wrapper
    return renderer



# coding: utf-8

"""   
    unittest base
    ~~~~~~~~~~~~~
    
    :copyleft: 2009 by the django-dbpreferences team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys
import unittest
from HTMLParser import HTMLParseError

from django.core import management
from django.utils.encoding import smart_str
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.utils.unittest.util import safe_repr

from django_tools.unittest_utils.html import parse_html

from BrowserDebug import debug_response
import difflib



class BaseTestCase(unittest.TestCase):
    # Should we open a bwoser traceback?
    browser_traceback = True

    TEST_USERS = {
        "superuser": {
            "username": "superuser",
            "email": "superuser@example.org",
            "password": "superuser_password",
            "is_staff": True,
            "is_superuser": True,
        },
        "staff": {
            "username": "staff test user",
            "email": "staff_test_user@example.org",
            "password": "staff_test_user_password",
            "is_staff": True,
            "is_superuser": False,
        },
        "normal": {
            "username": "normal test user",
            "email": "normal_test_user@example.org",
            "password": "normal_test_user_password",
            "is_staff": False,
            "is_superuser": False,
        },
    }

    def _pre_setup(self):
        super(BaseTestCase, self)._pre_setup()

    def login(self, usertype):
        """
        Login the user defined in self.TEST_USERS
        return User model instance
        """
        test_user = self._get_userdata(usertype)

        ok = self.client.login(username=test_user["username"],
                               password=test_user["password"])
        self.failUnless(ok, "Can't login test user '%s'!" % usertype)
        return self._get_user(usertype)

    def add_user_permissions(self, user, permissions):
        """
        add permissions to the given user instance.
        permissions e.g.: ("AppLabel.add_Modelname", "auth.change_user")
        """
        assert(permissions, (list, tuple))
        for permission in permissions:
            # permission, e.g: blog.add_blogentry
            app_label, permission_codename = permission.split(".", 1)
            model_name = permission_codename.split("_", 1)[1]

            try:
                content_type = ContentType.objects.get(app_label=app_label, model=model_name)
            except ContentType.DoesNotExist:
                etype, evalue, etb = sys.exc_info()
                evalue = etype("Can't get ContentType for app %r and model %r: %s" % (
                    app_label, model_name, evalue
                ))
                raise etype, evalue, etb


            perm = Permission.objects.get(content_type=content_type, codename=permission_codename)
            user.user_permissions.add(perm)
            user.save()

        self.assertTrue(user.has_perms(permissions))

    def _get_userdata(self, usertype):
        """ return userdata from self.TEST_USERS for the given usertype """
        try:
            return self.TEST_USERS[usertype]
        except KeyError, err:
            etype, evalue, etb = sys.exc_info()
            evalue = etype(
                "Wrong usetype %s! Existing usertypes are: %s" % (err, ", ".join(self.TEST_USERS.keys()))
            )
            raise etype, evalue, etb

    def _get_user(self, usertype):
        """ return User model instance for the goven usertype"""
        test_user = self._get_userdata(usertype)
        return User.objects.get(username=test_user["username"])

    def _create_testusers(self):
        """ Create all available testusers. """
        def create_user(username, password, email, is_staff, is_superuser):
            """
            Create a user and return the instance.
            """
            defaults = {'password':password, 'email':email}
            user, created = User.objects.get_or_create(
                username=username, defaults=defaults
            )
            if not created:
                user.email = email
            user.set_password(password)
            user.is_staff = is_staff
            user.is_superuser = is_superuser
            user.save()
        for usertype, userdata in self.TEST_USERS.iteritems():
            create_user(**userdata)

    def raise_browser_traceback(self, response, msg):
        debug_response(
            response, self.browser_traceback, msg, display_tb=False
        )
        msg += " (url: %r)" % response.request.get("PATH_INFO", "???")
        raise self.failureException, msg

    def assertStatusCode(self, response, excepted_code=200):
        """
        assert response status code, if wrong, do a browser traceback.
        """
        if response.status_code == excepted_code:
            return # Status code is ok.
        msg = "assertStatusCode error: %r != %r" % (response.status_code, excepted_code)
        self.raise_browser_traceback(response, msg)

    def assertRedirect(self, response, url, status_code=302):
        """
        assert than response is a redirect to the right destination, if wrong, do a browser traceback.
        """
        self.assertStatusCode(response, excepted_code=status_code)
        try:
            location = response['Location']
        except KeyError, err:
            self.raise_browser_traceback(response, "No 'Location' in response: %s" % err)
        else:
            if location != url:
                self.raise_browser_traceback(response, "Wrong destination url: %r != %r" % (location, url))

    def _assert_and_parse_html(self, html, user_msg, msg):
        """
        convert a html snippet into a DOM tree.
        raise error if snippet is no valid html.
        """
        try:
            return parse_html(html)
        except HTMLParseError, e:
            self.fail("html code is not valid: %s - code: %r" % (e, html))

    def _assert_and_parse_html_response(self, response):
        """
        convert html response content into a DOM tree.
        raise browser traceback, if content is no valid html.
        """
        try:
            return parse_html(response.content)
        except HTMLParseError, e:
            self.raise_browser_traceback(response, "Response's content is no valid html: %s" % e)

    def assertHTMLEqual(self, html1, html2, msg=None):
        """
        Asserts that two html snippets are semantically the same, e.g. whitespace
        in most cases is ignored, attribute ordering is not significant. The
        passed in arguments must be valid HTML.
        """
        dom1 = self._assert_and_parse_html(html1, msg,
            u'First argument is no valid html:')
        dom2 = self._assert_and_parse_html(html2, msg,
            u'Second argument is no valid html:')

        if dom1 != dom2:
            standardMsg = '%s != %s' % (safe_repr(dom1, True), safe_repr(dom2, True))
            diff = ('\n' + '\n'.join(difflib.ndiff(
                           unicode(dom1).splitlines(),
                           unicode(dom2).splitlines())))
            standardMsg = self._truncateMessage(standardMsg, diff)
            self.fail(self._formatMessage(msg, standardMsg))

    def assertContains(self, response, text, count=None, status_code=200,
                       msg_prefix='', html=False):
        """
        Asserts that a response indicates that some content was retrieved
        successfully, (i.e., the HTTP status code was as expected), and that
        ``text`` occurs ``count`` times in the content of the response.
        If ``count`` is None, the count doesn't matter - the assertion is true
        if the text occurs at least once in the response.
        """
        if msg_prefix:
            msg_prefix += ": "

        self.assertEqual(response.status_code, status_code,
            msg_prefix + "Couldn't retrieve content: Response code was %d"
            " (expected %d)" % (response.status_code, status_code))
        text = smart_str(text, response._charset)
        content = response.content
        if html:
            content = self._assert_and_parse_html(content, None,
                u'Response\'s content is no valid html:')
            text = self._assert_and_parse_html(text, None,
                u'Second argument is no valid html:')
        real_count = content.count(text)
        if count is not None:
            self.assertEqual(real_count, count,
                msg_prefix + "Found %d instances of '%s' in response"
                " (expected %d)" % (real_count, text, count))
        else:
            self.assertTrue(real_count != 0,
                msg_prefix + "Couldn't find '%s' in response" % text)

    def assertDOM(self, response, must_contain=(), must_not_contain=(), use_browser_traceback=True):
        """
        Asserts that html response contains 'must_contain' nodes, but no
        nodes from must_not_contain.      
        """
        def count_snippet(snippet, response_dom):
            snippet = smart_str(snippet, response._charset)
            txt_dom = self._assert_and_parse_html(
                snippet, None, u'Unittest snippet is no valid html:'
            )
            return response_dom.count(txt_dom)

        response_dom = self._assert_and_parse_html_response(response)

        for txt in must_contain:
            if count_snippet(txt, response_dom) == 0:
                msg = "HTML not in response: '%s'" % txt
                if use_browser_traceback:
                    self.raise_browser_traceback(response, msg)
                else:
                    self.fail(msg)


        for txt in must_not_contain:
            if count_snippet(txt, response_dom) > 0:
                msg = "HTML should not be in response: '%s'" % txt
                if use_browser_traceback:
                    self.raise_browser_traceback(response, msg)
                else:
                    self.fail(msg)

    def assertResponse(self, response, must_contain=(), must_not_contain=()):
        """
        Check the content of the response
        must_contain - a list with string how must be exists in the response.
        must_not_contain - a list of string how should not exists.
        """
        def count_snippet(snippet):
            snippet = smart_str(snippet, response._charset)
            return response.content.count(snippet)

        for txt in must_contain:
            if count_snippet(txt) == 0:
                self.raise_browser_traceback(response, "Text not in response: '%s'" % txt)

        for txt in must_not_contain:
            if count_snippet(txt) > 0:
                self.raise_browser_traceback(response, "Text should not be in response: '%s'" % txt)


def direct_run(raw_filename):
    """
    Run a test direct from a unittest file.
    A unittest file should add something like this:
    
    if __name__ == "__main__":
        # Run this unitest directly
        direct_run(__file__)
    """
    appname = os.path.splitext(os.path.basename(raw_filename))[0]
    print "direct run %r" % appname
    management.call_command('test', appname)




# coding: utf-8
"""
    models utils
    ~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author:$

    :copyleft: 2009 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.conf import settings
from django.db.models import signals
from django.utils.text import get_text_list
from django.db import connection, IntegrityError
from django.utils.translation import ugettext as _
from django.db.models.fields import FieldDoesNotExist

def check_unique_together(sender, **kwargs):
    """
    Check models unique_together manually. Django enforced unique together only the database level, but
    some databases (e.g. SQLite) doesn't support this.
    
    usage:
        from django.db.models import signals
        signals.pre_save.connect(check_unique_together, sender=MyModelClass)
        
    or use auto_add_check_unique_together(), see below.
    """
    instance = kwargs["instance"]
    for field_names in sender._meta.unique_together:
        model_kwargs = {}
        for field_name in field_names:
            try:
                data = getattr(instance, field_name)
            except FieldDoesNotExist:
                # e.g.: a missing field, which is however necessary.
                # The real exception on model creation should be raised. 
                continue
            model_kwargs[field_name] = data

        query_set = sender.objects.filter(**model_kwargs)
        if instance.pk != None:
            # Exclude the instance if it was saved in the past
            query_set = query_set.exclude(pk=instance.pk)

        count = query_set.count()
        if count > 0:
            field_names = get_text_list(field_names, _('and'))
            msg = _(u"%(model_name)s with this %(field_names)s already exists.") % {
                'model_name': unicode(instance.__class__.__name__),
                'field_names': unicode(field_names)
            }
            raise IntegrityError(msg)

def auto_add_check_unique_together(model_class):
    """
    Add only the signal handler check_unique_together, if a database without UNIQUE support is used.
    """
    if settings.DATABASE_ENGINE in ('sqlite3',): # 'postgresql', 'mysql', 'sqlite3' or 'ado_mssql'.
        signals.pre_save.connect(check_unique_together, sender=model_class)



# coding: utf-8

import os
import time
import warnings
import subprocess


__version__ = (0, 18, 1)


VERSION_STRING = '.'.join(str(part) for part in __version__)

#VERBOSE = True
VERBOSE = False

def _error(msg):
    if VERBOSE:
        warnings.warn(msg)
    return ""

def get_commit_timestamp(path=None):
    if path is None:
        path = os.path.abspath(os.path.dirname(__file__))

    try:
        process = subprocess.Popen(
            # %ct: committer date, UNIX timestamp  
            ["/usr/bin/git", "log", "--pretty=format:%ct", "-1", "HEAD"],
            shell=False, cwd=path,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        )
    except Exception, err:
        return _error("Can't get git hash: %s" % err)

    process.wait()
    returncode = process.returncode
    if returncode != 0:
        return _error(
            "Can't get git hash, returncode was: %r"
            " - git stdout: %r"
            " - git stderr: %r"
            % (returncode, process.stdout.readline(), process.stderr.readline())
        )

    output = process.stdout.readline().strip()
    try:
        timestamp = int(output)
    except Exception, err:
        return _error("git log output is not a number, output was: %r" % output)

    try:
        return time.strftime(".%m%d", time.gmtime(timestamp))
    except Exception, err:
        return _error("can't convert %r to time string: %s" % (timestamp, err))


VERSION_STRING += get_commit_timestamp()

if __name__ == "__main__":
    print VERSION_STRING



#coding:utf-8

"""
    Addon for django-tagging
    
    http://code.google.com/p/django-tagging/
    
    Some code parts from http://code.google.com/p/django-tagging-autocomplete/    
    
    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from tagging.forms import TagField
from django_tools.tagging_addon.widgets import jQueryTagFieldWidget

class jQueryTagFormField(TagField):
    """
    A form field for a django-tagging field.
    Use a own widget to display existing tags and make them clickable with jQuery.
    
    Needs as first initial argument the model class for the own widget.
    """
    def __init__(self, tag_model, *args, **kwargs):
        self.tag_model = tag_model
        kwargs["widget"] = jQueryTagFieldWidget(tag_model)
        super(jQueryTagFormField, self).__init__(*args, **kwargs)



# coding: utf-8


"""
    direcotory selection
    ~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import os

if __name__ == "__main__":
    # For doctest only
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django import forms
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from django_tools.utils.messages import failsafe_message
from django_tools import validators


class DirectoryWidget(forms.TextInput):
    """
    TODO: Add AJAX Stuff for easy select a existing directory path.
    """
    pass


class DirectoryFormField(forms.CharField):
    def __init__(self, base_path=settings.MEDIA_ROOT, *args, **kwargs):
        super(DirectoryFormField, self).__init__(*args, **kwargs)
        self.validators.append(
            validators.ExistingDirValidator(base_path=base_path)
        )

    def clean(self, value):
        value = super(DirectoryFormField, self).clean(value)
        value = os.path.normpath(value)
        return value


class DirectoryModelField(models.CharField):
    """
    >>> dir = DirectoryModelField()
    >>> dir.run_validators(settings.MEDIA_ROOT)
    >>> dir.run_validators("does/not/exist")
    Traceback (most recent call last):
        ...
    ValidationError: [u"Directory doesn't exist!"]
    
    >>> dir.run_validators("../")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Directory is not in base path!']
    
    >>> dir = DirectoryModelField(base_path="/")
    >>> dir.run_validators("/etc/default/")
    >>> dir.run_validators("var/log")
    >>> dir.run_validators("../bullshit")
    Traceback (most recent call last):
        ...
    ValidationError: [u"Directory doesn't exist!"]
    """
    __metaclass__ = models.SubfieldBase
    default_validators = []
    description = _("A existing/accessible directory")

    def __init__(self, max_length=256, base_path=settings.MEDIA_ROOT, *args, **kwargs):
        super(DirectoryModelField, self).__init__(*args, max_length=max_length, **kwargs)
        self.validators.append(
            validators.ExistingDirValidator(base_path=base_path)
        )

    def formfield(self, **kwargs):
        """ Use always own widget and form field. """
        kwargs["widget"] = DirectoryWidget
        kwargs['form_class'] = DirectoryFormField
        return super(DirectoryModelField, self).formfield(**kwargs)


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding:utf-8

"""
    Limit to usergroups
    ~~~~~~~~~~~~~~~~~~~
    
    Helper to limit something to:
        * for everyone (anonymous users)
        * for staff users
        * for superusers
        * for a user group
        
    In forms you will get a select field with:
        * anonymous users
        * staff users
        * superusers
        * ..all existing user groups..
        
        
    usage example
    ~~~~~~~~~~~~~
    
    model.py
    ---------------------------------------------------------------------------
    class FooBar(models.Model):
        permit_edit = limit_to_usergroups.UsergroupsModelField()
        permit_view = limit_to_usergroups.UsergroupsModelField()
    ---------------------------------------------------------------------------
    
    views.py
    ---------------------------------------------------------------------------
    def view(request):
        queryset = FooBar.objects.all()
        queryset = limit_to_usergroups.filter_permission(queryset, permit_view=request.user)
        ...
        
    def edit(request, id):
        foo = FooBar.objects.get(id=id)
        
        if not limit_to_usergroups.has_permission(poll, permit_edit=request.user):
            msg = _("You have no permission to edit this.")
            if settings.DEBUG:
                verbose_limit_name = limit_to_usergroups.get_verbose_limit_name(foo.permit_edit)
                msg += " (Limited to: %s)" % verbose_limit_name
            messages.error(request, msg)
            return HttpResponseRedirect("foobar)
        ...
    ---------------------------------------------------------------------------

    
    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, Group


def has_permission(item, **kwargs):
    """
    return True/False if use has permission stored in UsergroupsModelField.
    
    return True, if permission are ok
    return False, if permission are not sufficient.
    
    e.g.:
        has_permission(poll, permit_vote=request.user)
        has_permission(item, model_field1=user, model_field2=user)
    """
    for attr_name, user in kwargs.items():
        limit_permission_value = getattr(item, attr_name)
        limit_permission_value = int(limit_permission_value)

        if limit_permission_value == UsergroupsModelField.ANONYMOUS_USERS:
            continue

        if user.is_anonymous():
            return False

        if limit_permission_value == UsergroupsModelField.STAFF_USERS:
            if user.is_staff:
                continue
            else:
                return False

        if limit_permission_value == UsergroupsModelField.SUPERUSERS:
            if user.is_superuser:
                continue
            else:
                return False

        usergroup = Group.objects.get(id=limit_permission_value)
        usergroups = user.groups.all()
        if usergroup not in usergroups:
            return False

    return True



def filter_permission(queryset, **kwargs):
    """
    Filter a given queryset with UsergroupsModelField.
    
    e.g.:
        queryset = filter_permission(queryset, permit_view=request.user)
        queryset = filter_permission(queryset, model_field1=user, model_field2=user)
    
    FIXME: Would be nice, if this is a normal QuerySet method
    """
    result = [item for item in queryset if has_permission(item, **kwargs)]
    return result


def get_verbose_limit_name(value):
    """
    Simply convert the integer value of a UsergroupsModelField to his select choice text
    
    >>> get_verbose_limit_name(-1)
    u"staff users"
    """
    limit_dict = get_limit_dict()
    return limit_dict[value]


def get_user_groups():
    groups = Group.objects.values_list("id", "name")
    return list(groups)


def get_limit_dict():
    # use unicode() to evaluate ugettext_lazy:
    limit_dict = dict([(k, unicode(v)) for k, v in UsergroupsModelField.USER_TYPES_DICT.items()])

    groups = get_user_groups()
    limit_dict.update(dict(groups))
    return limit_dict


class UsergroupsModelField(models.IntegerField):
    """  
    TODO: Use html select optgroup [1] to group anonymous, staff and superusers
    from user groups
        [1] http://www.w3.org/wiki/HTML/Elements/optgroup
    """
    ANONYMOUS_USERS = 0
    STAFF_USERS = -1
    SUPERUSERS = -2

    USER_TYPES_CHOICES = [
        (ANONYMOUS_USERS, _("anonymous users")),
        (STAFF_USERS, _("staff users")),
        (SUPERUSERS, _("superusers")),
    ]
    USER_TYPES_DICT = dict(USER_TYPES_CHOICES)

    def get_choices(self, *args, **kwargs):
        groups = get_user_groups()
        choices = self.USER_TYPES_CHOICES + list(groups)
        return choices





# coding:utf-8

"""
    Client storage
    ~~~~~~~~~~~~~~
    
    Store data in a client cookie with a security hash.
    
    Usage e.g.:
    --------------------------------------------------------------------------
    from django_tools.utils.client_storage import ClientCookieStorage
    
    def view1(request):
        response = HttpResponse("Hello World!")
        c = ClientCookieStorage(cookie_key="my_key")
        response = c.save_data(my_data_string, response)
        return response
    
    def view2(request):
        c = ClientCookieStorage(cookie_key="my_key")
        data = c.get_data(request)
        if data is not None:
           ...do something...
    --------------------------------------------------------------------------
    
    :copyleft: 2010 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.conf import settings
from django.utils.hashcompat import sha_constructor
from django.middleware import csrf


SALT_LEN = 5 # length of the random salt value
HASH_LEN = 40 # length of a SHA-1 hexdigest


def get_new_salt():
    """
    >>> len(get_new_salt()) == SALT_LEN
    True
    """
    return csrf._get_new_csrf_key()[:SALT_LEN]


class InvalidCookieData(Exception):
    pass

class ClientCookieStorage(object):
    """
    >>> sha_constructor("foo").hexdigest()
    '0beec7b5ea3f0fdbc95d0dd47f3c5bc275da8a33'
    >>> len(sha_constructor("foo").hexdigest()) == HASH_LEN
    True
    >>> c = ClientCookieStorage("foo")
    >>> d = c._append_security_hash("bar")
    >>> c._decode_data(d) == "bar"
    True
    
    >>> c._decode_data("invalid")
    Traceback (most recent call last):
    ...
    InvalidCookieData: Salt/Hash wrong length!
    
    >>> c._decode_data("many invalid data without salt/hash information.")
    Traceback (most recent call last):
    ...
    InvalidCookieData: Salt/Hash inaccurate: invalid literal for int() with base 16: 'y inv'
    
    >>> c._decode_data("invalid123451234567890123456789012345678901234567890")
    Traceback (most recent call last):
    ...
    InvalidCookieData: Hash compare failed!
    """
    def __init__(self, cookie_key):
        self.cookie_key = cookie_key
        
    def _append_security_hash(self, data):
        """ append security hash to the data. """
        salt = get_new_salt()
        hash = sha_constructor(data + salt + settings.SECRET_KEY).hexdigest()
        data += salt + hash
        return data
    
    def _decode_data(self, data):
        """ return the decoded data, but only, if security hash is valid. """
        hash = data[-HASH_LEN:]
        salt = data[-(SALT_LEN+HASH_LEN):-HASH_LEN]
        data = data[:-(SALT_LEN+HASH_LEN)]
        
        if len(salt)!=SALT_LEN or len(hash)!=HASH_LEN:
            raise InvalidCookieData("Salt/Hash wrong length!")
        
        try:
            int(salt, 16)
            int(hash, 16)
        except ValueError, err:
            raise InvalidCookieData("Salt/Hash inaccurate: %s" % err)
        
        hash2 = sha_constructor(data + salt + settings.SECRET_KEY).hexdigest()
        if hash != hash2:
            raise InvalidCookieData("Hash compare failed!")
            
        return data
    
    def save_data(self, data, response, max_age = 60 * 60 * 24 * 7 * 52):
        """ Add a cookie to response object, with data and security hash. """
        data = self._append_security_hash(data)
        response.set_cookie(key=self.cookie_key, value=data, max_age = max_age)
        return response
        
    def get_data(self, request):
        """ Return the stored data from cookie, if security hash is valid. """
        if self.cookie_key not in request.COOKIES:
            return
        
        raw_data = request.COOKIES[self.cookie_key]
        data = self._decode_data(raw_data)
        return data



if __name__ == "__main__":
    import doctest
    doctest.testmod(verbose=False)
    print "DocTest end."


# coding: utf-8

"""
    models stuff
    ~~~~~~~~~~~~

    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.db import models
from django.contrib.auth.models import User


from django_tools.middlewares import ThreadLocal




class UpdateInfoBaseModel(models.Model):
    """
    Base model with update info attributes, used by many models.
    The createby and lastupdateby ForeignKey would be automaticly updated. This needs the 
    request object as the first argument in the save method.
    """
    objects = models.Manager()

    createtime = models.DateTimeField(auto_now_add=True, help_text="Create time")
    lastupdatetime = models.DateTimeField(auto_now=True, help_text="Time of the last change.")

    createby = models.ForeignKey(User, editable=False, related_name="%(class)s_createby",
        null=True, blank=True, # <- If the model used outsite a real request (e.g. unittest, db shell)
        help_text="User how create this entry.")
    lastupdateby = models.ForeignKey(User, editable=False, related_name="%(class)s_lastupdateby",
        null=True, blank=True, # <- If the model used outsite a real request (e.g. unittest, db shell)
        help_text="User as last edit this entry.")

    def save(self, *args, **kwargs):
        """
        Automatic update createby and lastupdateby attributes with the request object witch must be
        the first argument.
        """
        current_user = ThreadLocal.get_current_user()

        if current_user and isinstance(current_user, User):
            if self.pk == None or kwargs.get("force_insert", False): # New model entry
                self.createby = current_user
            self.lastupdateby = current_user

        return super(UpdateInfoBaseModel, self).save(*args, **kwargs)

    class Meta:
        abstract = True




# coding:utf-8

if __name__ == "__main__":
    print "run doctest:"
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"
    from django.conf import global_settings
    global_settings.INSTALLED_APPS += (
        'django.contrib.admindocs',
        'django.contrib.auth',
        'django.contrib.flatpages',
    )


from django.conf import settings
from django.core import urlresolvers
from django.utils.importlib import import_module


def get_filtered_apps(resolve_url="/", no_args=True, debug=False, skip_fail=False):
    """
    Filter settings.INSTALLED_APPS and create a list
    of all Apps witch can resolve the given url >resolve_url<

    @param resolve_url: url used for RegexURLResolver
    @param no_args: Only views without args/kwargs ?
    @parm skip_fail: If True: raise excaption if app is not importable
    
    >>> get_filtered_apps()
    ['django.contrib.admindocs']
    
    >>> get_filtered_apps(no_args=False)
    ['django.contrib.admindocs', 'django.contrib.flatpages']
    
    >>> get_filtered_apps(debug=True)
    found 'django.contrib.admindocs' with urls.py
    found 'django.contrib.auth' with urls.py
    Skip 'django.contrib.auth': Can't handle root url.
    found 'django.contrib.flatpages' with urls.py
    ['django.contrib.admindocs']
    """
    root_apps = []
    for app_label in settings.INSTALLED_APPS:
        urls_pkg = "%s.urls" % app_label
        try:
            url_mod = import_module(urls_pkg)
        except ImportError, err:
            if debug:
                print "Skip %r: has no urls.py" % app_label
            if str(err) == "No module named urls":
                continue
            if not skip_fail:
                raise
        except Exception, err:
            if debug:
                print "Error importing %r: %s" % (app_label, err)
            if not skip_fail:
                raise
            else:
                continue


        if debug:
            print "found %r with urls.py" % app_label

        try:
            urlpatterns = url_mod.urlpatterns
        except AttributeError:
            if debug:
                print "Skip %r: urls.py has no 'urlpatterns'" % app_label
            continue

        resolver = urlresolvers.RegexURLResolver(r'^/', urlpatterns)
        try:
            func, func_args, func_kwargs = resolver.resolve(resolve_url)
        except urlresolvers.Resolver404, err:
            if debug:
                print "Skip %r: Can't handle root url." % app_label
            continue
        if not no_args or func_args == () and func_kwargs == {}:
            root_apps.append(app_label)
    return root_apps


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding: utf-8

"""
    media path selection
    ~~~~~~~~~~~~~~~~~~~~
    
    TODO: Made this generic and don't use settings.MEDIA_ROOT direct, let it
    be set as a argument to widget/fields etc.
    
     * model field
     * form field
     * widget

    :copyleft: 2010 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # For doctest only
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django import forms
from django.db import models
from django.conf import settings

from django_tools.utils.messages import failsafe_message


def directory_walk(path):
    """
    Directory tree generator
    similar to os.walk, except:
      - yield only directories
      - sorted list case-insensitive
      - follow links (os.walk can do this since python 2.6)
    """
    dirs = []
    for name in os.listdir(path):
        if os.path.isdir(os.path.join(path, name)):
            dirs.append(name)

    yield path

    # Sort case-insensitive
    dirs.sort(cmp=lambda x, y: cmp(x.lower(), y.lower()))

    for dir in dirs:
        sub_path = os.path.join(path, dir)
        for x in directory_walk(sub_path):
            yield x




class MediaPathWidget(forms.Select):
    """
    Select a sub directory in settings.MEDIA_ROOT
    
    >>> MediaPathWidget().choices # doctest: +ELLIPSIS
    [('/template', '/template'), ..., ('/utils', '/utils')]
    """
    def __init__(self, attrs=None):
        super(MediaPathWidget, self).__init__(attrs)

        self._base_path = os.path.abspath(os.path.normpath(settings.MEDIA_ROOT))

        try:
            self.choices = self._get_path_choices()
        except OSError, err:
            self.choices = []
            if settings.DEBUG:
                failsafe_message("Can't read MEDIA_ROOT: %s" % err)

    def _get_path_choices(self):
        media_dirs_choices = []
        cut_pos = len(self._base_path)
        for root in directory_walk(self._base_path):
            rel_dir = root[cut_pos:].strip(os.sep)
            if rel_dir:
                media_dirs_choices.append((rel_dir, rel_dir))
        return media_dirs_choices


class MediaPathModelField(models.TextField):
    """
    
    """
    __metaclass__ = models.SubfieldBase

#    def __init__(self, separator=",", strip_items=True, skip_empty=True, *args, **kwargs):
#        self.separator = separator
#        self.strip_items = strip_items
#        self.skip_empty = skip_empty
#        super(MediaPathModelField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        """ Use always own widget and form field. """
        kwargs["widget"] = MediaPathWidget
#        kwargs["form_class"] = SignSeparatedFormField
        return super(MediaPathModelField, self).formfield(**kwargs)


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



#!/usr/bin/env python
# coding: utf-8

"""
    distutils setup
    ~~~~~~~~~~~~~~~

    :copyleft: 2009-2010 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys

from setuptools import setup, find_packages

from django_tools import VERSION_STRING


PACKAGE_ROOT = os.path.dirname(os.path.abspath(__file__))


def get_authors():
    try:
        f = file(os.path.join(PACKAGE_ROOT, "AUTHORS"), "r")
        authors = [l.strip(" *\r\n") for l in f if l.strip().startswith("*")]
        f.close()
    except Exception, err:
        authors = "[Error: %s]" % err
    return authors


def get_long_description():
    try:
        f = file(os.path.join(PACKAGE_ROOT, "README.textile"), "r")
        long_description = f.read().strip()
        f.close()
    except Exception, err:
        long_description = "[Error: %s]" % err
    return long_description


setup(
    name='django-tools',
    version=VERSION_STRING,
    description='miscellaneous tools for django',
    long_description=get_long_description(),
    author=get_authors(),
    maintainer="Jens Diemer",
    maintainer_email="django-tools@jensdiemer.de",
    url='http://code.google.com/p/django-tools/',
    packages=find_packages(),
    include_package_data=True, # include package data under svn source control
    zip_safe=False,
    classifiers=[
#        "Development Status :: 4 - Beta",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
#        "Intended Audience :: Education",
#        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        'Framework :: Django',
        "Topic :: Database :: Front-Ends",
        "Topic :: Documentation",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Operating System :: OS Independent",
    ]
)



# coding:utf-8


"""
    url form/model field
    ~~~~~~~~~~~~~~~~~~~~
    
    flexible URL form and model field used own URLValidator2.

    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"


from django_tools.validators import URLValidator2

from django.core import validators
from django.db.models.fields import CharField as OriginModelCharField
from django.forms.fields import CharField as OriginFormsCahrField
from django.utils.translation import ugettext_lazy as _


class URLFormField2(OriginFormsCahrField):
    """
    A flexible version of the original django form URLField ;)
    
    Please read the django_tools.validators.URLValidator2 DocString, too!
    
    
    To allow only local domains, disallow scheme and netloc:
    >>> URLFormField2(allow_schemes=None, allow_netloc=False).clean("/path/?query#fragment")
    u'/path/?query#fragment'
    >>> URLFormField2(allow_schemes=None, allow_netloc=False).clean("http://domain.tld/path/?query#fragment")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Please enter a local URL (without protocol/domain).']
    
    **Note:** this is also a valid "local" path (more info in URLValidator2 DocString):
    >>> URLFormField2(allow_schemes=None, allow_netloc=False).clean("domain.tld/path/?query#fragment")
    u'domain.tld/path/?query#fragment'
    
    
    >>> f = URLFormField2(verify_exists=True)
    >>> f.clean("http://www.pylucid.org/") # Failed if pylucid.org is down ;)
    u'http://www.pylucid.org/'
    >>> f.clean("http://www.domain.tld/valid/url/does/not/exist/")
    Traceback (most recent call last):
        ...
    ValidationError: [u'This URL appears to be a broken link.']
    
    
    >>> URLFormField2(allow_schemes=("svn",)).clean("svn://domain.tld")
    u'svn://domain.tld'
    >>> URLFormField2(allow_schemes=("http","ftp")).clean("svn://domain.tld")
    Traceback (most recent call last):
        ...
    ValidationError: [u"The URL doesn't start with a allowed scheme."]
    """
    default_error_messages = {
        'invalid': _(u'Enter a valid URL.'),
        'invalid_link': _(u'This URL appears to be a broken link.'),
    }

    def __init__(self, max_length=None, min_length=None, verify_exists=False,
            allow_schemes=("http", "https"), allow_all_schemes=False, allow_netloc=True,
            allow_query=True, allow_fragment=True,
            validator_user_agent=validators.URL_VALIDATOR_USER_AGENT, *args, **kwargs):

        super(URLFormField2, self).__init__(max_length, min_length, *args, **kwargs)

        self.validators.append(
            URLValidator2(verify_exists=verify_exists,
                allow_schemes=allow_schemes, allow_all_schemes=allow_all_schemes, allow_netloc=allow_netloc,
                allow_query=allow_query, allow_fragment=allow_fragment,
                validator_user_agent=validator_user_agent
            )
        )



class URLModelField2(OriginModelCharField):
    """
    A flexible version of the original django model URLField ;)
    
    Please read the django_tools.validators.URLValidator2 DocString, too!
    
    
    >>> f = URLModelField2(verify_exists=False).formfield()
    >>> isinstance(f, URLFormField2)
    True
    >>> f.clean(u"http://www.domain.tld/path?query#fragment")
    u'http://www.domain.tld/path?query#fragment'
    
    >>> f = URLModelField2().formfield()
    >>> f.clean("svn://domain.tld")
    Traceback (most recent call last):
        ...
    ValidationError: [u"The URL doesn't start with a allowed scheme."]
    
    >>> f = URLModelField2(verify_exists=True).formfield()
    >>> f.clean("http://www.pylucid.org/") # Failed if pylucid.org is down ;)
    u'http://www.pylucid.org/'
    >>> f.validators[1].verify_exists
    True
    >>> f.clean("http://www.domain.tld/valid/url/does/not/exist/")
    Traceback (most recent call last):
        ...
    ValidationError: [u'This URL appears to be a broken link.']
    
    >>> f = URLModelField2(verify_exists=False, allow_query=False).formfield()
    >>> f.clean("http://www.domain.tld/without/query/")
    u'http://www.domain.tld/without/query/'
    >>> f.clean("http://www.domain.tld/with/?query")
    Traceback (most recent call last):
        ...
    ValidationError: [u'Enter a valid URL without a query.']
    
    """
    description = _("URL")

    def __init__(self, verbose_name=None, name=None, verify_exists=True,
            allow_schemes=("http", "https"), allow_all_schemes=False, allow_netloc=True,
            allow_query=True, allow_fragment=True,
            validator_user_agent=validators.URL_VALIDATOR_USER_AGENT, **kwargs):

        kwargs['max_length'] = kwargs.get('max_length', 200)
        OriginModelCharField.__init__(self, verbose_name, name, **kwargs)

        self.verify_exists = verify_exists
        self.validator_user_agent = validator_user_agent

        self.allow_schemes = allow_schemes or ()
        self.allow_all_schemes = allow_all_schemes
        self.allow_netloc = allow_netloc
        self.allow_query = allow_query
        self.allow_fragment = allow_fragment

        self.validators.append(
            URLValidator2(verify_exists=verify_exists,
                allow_schemes=allow_schemes, allow_all_schemes=allow_all_schemes, allow_netloc=allow_netloc,
                allow_query=allow_query, allow_fragment=allow_fragment,
                validator_user_agent=validator_user_agent
            )
        )

    def formfield(self, **kwargs):
        # As with CharField, this will cause URL validation to be performed twice
        defaults = {
            'form_class': URLFormField2,

            "verify_exists":self.verify_exists,
            "validator_user_agent": self.validator_user_agent,

            "allow_schemes":self.allow_schemes,
            "allow_all_schemes":self.allow_all_schemes,
            "allow_netloc":self.allow_netloc,
            "allow_query":self.allow_query,
            "allow_fragment":self.allow_fragment,
        }
        defaults.update(kwargs)
        return super(URLModelField2, self).formfield(**defaults)



if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



#!/usr/bin/env python
# coding: utf-8

"""
    stack information
    ~~~~~~~~~~~~~~~~~

    :copyleft: 2011 by the django-tools team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import inspect


STACK_LIMIT = 6 # Display only the last X stack lines
MAX_FILEPATH_LEN = 50 # Cut filepath in stack info message


def format_list(extracted_list):
    """
    Format a list of traceback entry tuples.
    """
    list = []
    for _, filename, lineno, func_name, line, _  in extracted_list:
        code = "".join(line).strip()
        item = (
            'File "%s", line %d, in %s\n'
            '  %s\n'
        ) % (filename, lineno, func_name, code)
        list.append(item)
    return list


def get_stack_info(filepath_filter, stack_limit=STACK_LIMIT, max_filepath_len=MAX_FILEPATH_LEN):
    """
    return stack_info: Where from the announcement comes?
    """
    stack_list = inspect.stack()
    stack_list.reverse()

    # go forward in the stack, till outside of this file.
    for no, stack_line in enumerate(stack_list):
        filepath = stack_line[1]
        if filepath_filter in filepath:
            break

    # Display only the last entries, till outside of this file
    stack_list = stack_list[:no]
    stack_list = stack_list[-stack_limit:]

    stack_info = format_list(stack_list)
    return stack_info
